import { Component, Input, OnInit } from '@angular/core';
import { HttpClient } from '@angular/common/http';

interface Partition {
  size: number;
  type: string;
  sizeBytes: number;
  format: boolean;
  color: string;
  percentage: number;
}

@Component({
  selector: 'app-partition-assistant',
  templateUrl: './partition-assistant.component.html',
  styleUrls: ['./partition-assistant.component.css']
})
export class PartitionAssistantComponent implements OnInit {
  @Input() data: any;
  @Input() clientUuid: string | undefined;  // El clientUuid que necesitas

  errorMessage = '';
  originalPartitions: any[] = [];
  disks: { diskNumber: number, totalDiskSize: number, partitions: Partition[] }[] = [];

  private apiUrl = 'http://127.0.0.1:8001/partitions';

  constructor(private http: HttpClient) {}

  ngOnInit() {
    this.initializeDisks();
  }

  initializeDisks() {
    const partitionsFromData = this.data.partitions;
    this.originalPartitions = JSON.parse(JSON.stringify(partitionsFromData)); // Guardar una copia de las particiones originales

    const disksMap = new Map<number, { totalDiskSize: number, partitions: Partition[] }>();

    partitionsFromData.forEach((partition: any) => {
      if (!disksMap.has(partition.diskNumber)) {
        disksMap.set(partition.diskNumber, { totalDiskSize: 0, partitions: [] });
      }

      const disk = disksMap.get(partition.diskNumber);
      if (partition.partitionNumber === 0) {
        disk!.totalDiskSize = this.convertBytesToGB(partition.size);
      } else {
        disk!.partitions.push({
          size: this.convertBytesToGB(partition.size),
          type: 'NTFS',  // Puedes cambiar el tipo según sea necesario
          sizeBytes: partition.size,
          format: false,
          color: '#' + Math.floor(Math.random() * 16777215).toString(16),
          percentage: 0
        });
      }
    });

    disksMap.forEach((disk, diskNumber) => {
      this.updatePartitionPercentages(disk.partitions, disk.totalDiskSize);
      this.disks.push({
        diskNumber: diskNumber,
        totalDiskSize: disk.totalDiskSize,
        partitions: disk.partitions
      });
    });
  }

  convertBytesToGB(bytes: number): number {
    return bytes / (1024 * 1024 * 1024);
  }

  updatePartitionPercentages(partitions: Partition[], totalDiskSize: number) {
    partitions.forEach(partition => {
      partition.percentage = (partition.size / totalDiskSize) * 100;
    });
  }

  // Añadir una nueva partición
  addPartition(diskNumber: number) {
    const disk = this.disks.find(d => d.diskNumber === diskNumber);

    if (disk) {
      const remainingGB = this.getRemainingGB(disk.partitions, disk.totalDiskSize);
      if (remainingGB > 0) {
        disk.partitions.push({
          size: 0,
          type: 'NTFS', // Tipo por defecto, puede ser cambiado por el usuario
          sizeBytes: 0,
          format: false,
          color: '#' + Math.floor(Math.random() * 16777215).toString(16), // Color aleatorio
          percentage: 0
        });
        this.updatePartitionPercentages(disk.partitions, disk.totalDiskSize);
      } else {
        this.errorMessage = 'No hay suficiente espacio libre en el disco para crear una nueva partición.';
      }
    }
  }

  // Actualizar el tamaño de una partición
  updatePartitionSize(diskNumber: number, index: number, size: number) {
    const disk = this.disks.find(d => d.diskNumber === diskNumber);
    if (disk) {
      const partition = disk.partitions[index];
      const remainingGB = this.getRemainingGB(disk.partitions, disk.totalDiskSize) + partition.size;

      if (size > remainingGB) {
        this.errorMessage = `El tamaño de la partición no puede superar el espacio libre (${remainingGB.toFixed(2)} GB).`;
      } else {
        this.errorMessage = '';
        partition.size = size;
        partition.sizeBytes = size * 1024 * 1024 * 1024;
        this.updatePartitionPercentages(disk.partitions, disk.totalDiskSize);
      }
    }
  }

  getRemainingGB(partitions: Partition[], totalDiskSize: number): number {
    const totalUsedGB = partitions.reduce((acc, partition) => acc + partition.size, 0);
    return Math.max(0, totalDiskSize - totalUsedGB);
  }

  isPartitionModified(original: any, current: Partition): boolean {
    return (
      this.convertBytesToGB(original.size) !== current.size ||
      original.type !== current.type
    );
  }

  getModifiedOrNewPartitions() {
    const modifiedPartitions: any[] = [];

    this.disks.forEach(disk => {
      disk.partitions.forEach((partition, index) => {
        const originalPartition = this.originalPartitions.find(
          p => p.diskNumber === disk.diskNumber && p.partitionNumber === index + 1
        );

        // Si no existe en las originales, es nueva
        if (!originalPartition) {
          modifiedPartitions.push({ partition, diskNumber: disk.diskNumber, partitionNumber: index + 1 });
        } else if (this.isPartitionModified(originalPartition, partition)) {
          modifiedPartitions.push({ partition, diskNumber: disk.diskNumber, partitionNumber: index + 1 });
        }
      });
    });

    return modifiedPartitions;
  }

  save() {
    const modifiedPartitions = this.getModifiedOrNewPartitions();

    if (modifiedPartitions.length === 0) {
      this.errorMessage = 'No hay cambios para guardar.';
      return;
    }

    modifiedPartitions.forEach(({ partition, diskNumber, partitionNumber }) => {
      const payload = {
        diskNumber: diskNumber,
        partitionNumber: partitionNumber,
        size: partition.size,
        filesystem: partition.type,
        client: `https://example.com/${this.clientUuid}`
      };

      this.http.post(this.apiUrl, payload).subscribe(
        response => {
          console.log('Partición guardada exitosamente:', response);
        },
        error => {
          console.error('Error al guardar la partición:', error);
        }
      );
    });
  }

  // Eliminar partición de un disco
  removePartition(diskNumber: number, index: number) {
    const disk = this.disks.find(d => d.diskNumber === diskNumber);

    if (disk) {
      disk.partitions.splice(index, 1);
      this.updatePartitionPercentages(disk.partitions, disk.totalDiskSize);
    }
  }
}
